==> LL_utils-4.1.1.4-power-Linux-RHEL6.tar.gz: var/www/ll/cgi-bin/on_reconfig.pl <==
#!/usr/bin/perl

# static char sccsid[] = "@(#)86 1.9 src/ll/config_editor/cgi-bin/on_reconfig.pl, ll.config, ll_rrur2, rrur21042a 12/13/09 07:00:26\n";

use strict;
use lib "/var/www/ll/cgi-bin";
use LLUtil;
use DBI;

my ($mysql_host, $mysql_user, $mysql_pass, $mysql_db) = get_db_info(0);

my $query = $ENV{QUERY_STRING};
$query = unescape($query);

debug_log($query);

my $json_str = "Content-type: text/html\n\n";

# Machine-node01=1&MachineGroup-mg001=1&Cluster-cnbladel1n01_ggg=1&

my @keywords = split('&',$query);
my %machine_map = ();
my %machine_group_map = ();
my %cluster_map = ();
my @tmp;

my $dbh = DBI->connect("DBI:mysql:database=$mysql_db;host=$mysql_host",$mysql_user,$mysql_pass);
if(!defined($dbh))
{
	$json_str .= DBI->errstr;
	print $json_str;		
	exit 1;
}

# parse the request string.
foreach my $akey (@keywords)
{
	@tmp = split('=',$akey);
	my $id = $tmp[1];
	@tmp = split('-',$tmp[0]);
	if($tmp[0] eq "Machine")
	{

		if($tmp[1] eq "default" || $tmp[1] eq "Default")
		{
			my $sql_str="select TLL_Cluster.clusterID,TLL_Cluster.name from TLL_Nodelist,TLL_Cluster where TLL_Cluster.clusterID = TLL_Nodelist.clusterID && TLL_Nodelist.hostname = '".$tmp[1]."'";
			my $sth=$dbh->prepare($sql_str);
			if(defined($sth->execute()))
			{
				my($clusterID,$name);
				$sth->bind_columns(undef,\$clusterID,\$name);
				if($sth->fetch())
				{
					$cluster_map{$clusterID} = $name;
					debug_log("Find cluster info for default machine, cluster id - ".$clusterID.", cluster name - ".$name.".");
				}
				else
				{
					# xyyin: If we can not find the cluster for 'default' machine, use a fake cluster info.
					$cluster_map{1} = "Default";
					debug_log("Can not find cluster info for default machine, use default cluster instead.");
				}
			}
			else
			{
				$json_str .= $dbh->errstr;
				print $json_str;		
				exit 1;
			}
			$sth->finish();
		}
		else
		{
			$machine_map{$id} = $tmp[1];
		}
	}
	elsif($tmp[0] eq "MachineGroup")
	{
		$machine_group_map{$id} = $tmp[1];
	}
	elsif($tmp[0] eq "Cluster")
	{
		$cluster_map{$id} = $tmp[1];
	}
	else
	{
		debug_log("Error in on_reconfig.pl - invalid reconfig type: ".$tmp[0]."-".$tmp[1]."-".$id);
		exit 1;
	}
}

my %final_machine_map = ();
my %final_machine_group_map = ();

# if a machine which needs reconfiguration is in a machine group or a cluster which needs reconfiguration, we do not need to store it in the %final_machine_map.
foreach my $akey (keys %machine_map)
{
	my $sql_str="select clusterID,mgID from TLL_Nodelist where nodeID=".$akey;
	my $sth=$dbh->prepare($sql_str);
	if(defined($sth->execute()))
	{
		my($clusterID,$mgID);
		$sth->bind_columns(undef,\$clusterID,\$mgID);
		if($sth->fetch())
		{
			if( (!defined($machine_group_map{$mgID})) && (!defined($cluster_map{$clusterID})) )
			{
				$final_machine_map{$akey} = $machine_map{$akey};
			}
		}
		else
		{
			# clusterID is NULL or mgID is NULL
			$final_machine_map{$akey} = $machine_map{$akey};
		}
	}
	else
	{
		$json_str .= $dbh->errstr;
		print $json_str;		
		exit 1;
	}
	$sth->finish();
}

# if a machine group which needs reconfiguration is in a cluster which needs reconfiguration, we do not need to store it in the %final_machine_group_map.
foreach my $akey (keys %machine_group_map)
{
	my $sql_str="select clusterID from TLL_MachineGroup where mgID=".$akey;
	my $sth=$dbh->prepare($sql_str);
	if(defined($sth->execute()))
	{
		my($clusterID);
		$sth->bind_columns(undef,\$clusterID);
		if($sth->fetch())
		{
			if( !defined($cluster_map{$clusterID}) )
			{
				$final_machine_group_map{$akey} = $machine_group_map{$akey};
			}
		}
		else
		{
			# clusterID is NULL
			$final_machine_group_map{$akey} = $machine_group_map{$akey};
		}
	}
	else
	{
		$json_str .= $dbh->errstr;
		print $json_str;		
		exit 1;
	}	
	$sth->finish();
}

my $cmd_name = "/var/www/ll/cgi-bin/wrapper.cgi";
my $cluster_count = scalar(keys %cluster_map);
$? = 0;
if($cluster_count > 0)
{
	# xyyin: now we can not send reconfiguration command to a remote cluster.
	# so, if the %cluster_map is not empty, we reconfig the local cluster.

	# Get the admin name.
	my $admin_name;
	my $sql_str="select admin_list from TLL_CFGCluster limit 1";
	my $sth=$dbh->prepare($sql_str);
	if(defined($sth->execute()))
	{
		my($admin);
		$sth->bind_columns(undef,\$admin);
		while($sth->fetch())
		{
			my @tmp = split(' ',$admin);
			$admin_name = $tmp[0];
		}
	}
	else
	{
		$json_str .= $dbh->errstr;
		print $json_str;
		exit 1;
	}	

	# Change user to the admin and run reconfig
	debug_log("command:".$cmd_name." ".$admin_name." -g");
	my $out_str = `$cmd_name $admin_name -g`;
	if($? != 0)
	{
		$json_str .= $out_str;
		print $json_str;
		exit 1;
	}
}
else
{
	my @all_machines = ();
	
	# reconfig each machine in %final_machine_map
	foreach my $machID (keys %final_machine_map)
	{
		push(@all_machines,$final_machine_map{$machID});
		debug_log($machID);
		
	}

	# go through the %final_machine_group_map, and reconfig each machine in these machine groups
	foreach my $mgID (keys %final_machine_group_map)
	{
		my $sql_str="select hostname from TLL_Nodelist where mgID=".$mgID;
		my $sth=$dbh->prepare($sql_str);
		if(defined($sth->execute()))
		{
			my($hostname);
			$sth->bind_columns(undef,\$hostname);
			while($sth->fetch())
			{
				push(@all_machines,$hostname);
			}
		}
		else
		{
			$json_str .= $dbh->errstr;
			print $json_str;
			exit 1;
		}	
		$sth->finish();		
	}	

	foreach my $machine (@all_machines)
	{
		# Get the admin name.
		my $admin_name;
		my $sql_str="select admin_list from TLL_CFGCluster,TLL_Nodelist where TLL_CFGCluster.clusterID=TLL_Nodelist.clusterID and TLL_Nodelist.hostname='".$machine."'";
		my $sth=$dbh->prepare($sql_str);
		if(defined($sth->execute()))
		{
			my($admin);
			$sth->bind_columns(undef,\$admin);
			if($sth->fetch())
			{
				my @tmp = split(' ',$admin);
				$admin_name = $tmp[0];
				debug_log("Get admin name : ".$admin_name);
			}
			else
			{
				$json_str .= "Can not get the admin list for mchine - ".$machine;
				print $json_str;
				exit 1;
			}
		}
		else
		{
			$json_str .= $dbh->errstr;
			print $json_str;
			exit 1;
		}	

		# Change user to the admin and run reconfig
		my $out_str = `$cmd_name $admin_name -h $machine`;
		debug_log($out_str);
		
		if($? != 0)
		{
			debug_log($out_str);
			$json_str .= $out_str;
			print $json_str;
			exit 1;
		}
	}
	
}

$dbh->disconnect();
$json_str .= "OK";
print $json_str;

==> LL_utils-5.1.0.17-power-Linux-RHEL6.tar.gz: var/www/ll/cgi-bin/on_reconfig.pl <==
#!/usr/bin/perl

# static char sccsid[] = "@(#)86 1.12 src/ll/config_editor/cgi-bin/on_reconfig.pl, ll.config, ll_rrnep2, rrnep2s015a 3/27/13 17:34:19\n";

use strict;
use lib "/var/www/ll/cgi-bin";
use LLUtil;
use DBI;

my ($db_host, $db_user, $db_pass, $db_db, $db_conn, $db2inst) = get_db_info(0);

my $query = $ENV{QUERY_STRING};
$query = unescape($query);

debug_log($query);

my $json_str = "Content-type: text/html\n\n";

# Machine-node01=1&MachineGroup-mg001=1&Cluster-cnbladel1n01_ggg=1&

my @keywords = split('&',$query);
my %machine_map = ();
my %machine_group_map = ();
my %cluster_map = ();
my @tmp;

my $dbh = get_dbh(0);
if(!defined($dbh))
{
       $json_str .= DBI->errstr;
       print $json_str;
       exit 1;

#	error_and_exit($dbh->errstr);
}

# parse the request string.
foreach my $akey (@keywords)
{
	@tmp = split('=',$akey);
	my $id = $tmp[1];
	@tmp = split('-',$tmp[0]);
	if($tmp[0] eq "Machine")
	{

		if($tmp[1] eq "default" || $tmp[1] eq "Default")
		{
			my $sql_str="select TLL_Cluster.clusterID,TLL_Cluster.name from TLL_Nodelist,TLL_Cluster where TLL_Cluster.clusterID = TLL_Nodelist.clusterID and TLL_Nodelist.hostname = '".$tmp[1]."'";
			my $sth=$dbh->prepare($sql_str);
			if(defined($sth->execute()))
			{
				my($clusterID,$name);
				$sth->bind_columns(undef,\$clusterID,\$name);
				if($sth->fetch())
				{
					$cluster_map{$clusterID} = $name;
					debug_log("Find cluster info for default machine, cluster id - ".$clusterID.", cluster name - ".$name.".");
				}
				else
				{
					# xyyin: If we can not find the cluster for 'default' machine, use a fake cluster info.
					$cluster_map{1} = "Default";
					debug_log("Can not find cluster info for default machine, use default cluster instead.");
				}
			}
			else
			{
				$json_str .= $dbh->errstr;
				print $json_str;		
				exit 1;
			}
			$sth->finish();
		}
		else
		{
			$machine_map{$id} = $tmp[1];
		}
	}
	elsif($tmp[0] eq "MachineGroup")
	{
		$machine_group_map{$id} = $tmp[1];
	}
	elsif($tmp[0] eq "Cluster")
	{
		$cluster_map{$id} = $tmp[1];
	}
	else
	{
		debug_log("Error in on_reconfig.pl - invalid reconfig type: ".$tmp[0]."-".$tmp[1]."-".$id);
		exit 1;
	}
}

my %final_machine_map = ();
my %final_machine_group_map = ();

# if a machine which needs reconfiguration is in a machine group or a cluster which needs reconfiguration, we do not need to store it in the %final_machine_map.
foreach my $akey (keys %machine_map)
{
	my $sql_str="select clusterID,mgID from TLL_Nodelist where nodeID=".$akey;
	my $sth=$dbh->prepare($sql_str);
	if(defined($sth->execute()))
	{
		my($clusterID,$mgID);
		$sth->bind_columns(undef,\$clusterID,\$mgID);
		if($sth->fetch())
		{
			if( (!defined($machine_group_map{$mgID})) && (!defined($cluster_map{$clusterID})) )
			{
				$final_machine_map{$akey} = $machine_map{$akey};
			}
		}
		else
		{
			# clusterID is NULL or mgID is NULL
			$final_machine_map{$akey} = $machine_map{$akey};
		}
	}
	else
	{
		$json_str .= $dbh->errstr;
		print $json_str;		
		exit 1;
	}
	$sth->finish();
}

# if a machine group which needs reconfiguration is in a cluster which needs reconfiguration, we do not need to store it in the %final_machine_group_map.
foreach my $akey (keys %machine_group_map)
{
	my $sql_str="select clusterID from TLL_MachineGroup where mgID=".$akey;
	my $sth=$dbh->prepare($sql_str);
	if(defined($sth->execute()))
	{
		my($clusterID);
		$sth->bind_columns(undef,\$clusterID);
		if($sth->fetch())
		{
			if( !defined($cluster_map{$clusterID}) )
			{
				$final_machine_group_map{$akey} = $machine_group_map{$akey};
			}
		}
		else
		{
			# clusterID is NULL
			$final_machine_group_map{$akey} = $machine_group_map{$akey};
		}
	}
	else
	{
		$json_str .= $dbh->errstr;
		print $json_str;		
		exit 1;
	}	
	$sth->finish();
}

my $cmd_name = "/var/www/ll/cgi-bin/wrapper.cgi";
# For db2 only, export EXTSHM.  Is needed when the configurator runs in wrapper.
if ($db_conn =~ /^DB2/)
{
	$cmd_name = "export EXTSHM=ON; ".$cmd_name;	
}
my $cluster_count = scalar(keys %cluster_map);
$? = 0;
if($cluster_count > 0)
{
	# xyyin: now we can not send reconfiguration command to a remote cluster.
	# so, if the %cluster_map is not empty, we reconfig the local cluster.

	# Get the admin name.
	my $admin_name = get_admin();
	if (length($admin_name) == 0)
	{
		$json_str .= "No LoadLeveler admin is available to run the reconfiguration on this node.\nRun the reconfiguration from the command line.";
		print $json_str;
		exit 1;
	}
	# Change user to the admin and run reconfig
	debug_log("command:".$cmd_name." ".$admin_name." -g");
	my $out_str = `$cmd_name $admin_name -g`;
	if($? != 0)
	{
		debug_log("on_reconfig.pl: bad rc from command, out_str:".$out_str);
		$json_str .= $out_str;
		print $json_str;
		exit 1;
	}
	debug_log("on_reconfig.pl: good rc from command, out_str:".$out_str);
}
else
{
	my @all_machines = ();
	
	# reconfig each machine in %final_machine_map
	foreach my $machID (keys %final_machine_map)
	{
		push(@all_machines,$final_machine_map{$machID});
		debug_log($machID);
		
	}

	# go through the %final_machine_group_map, and reconfig each machine in these machine groups
	foreach my $mgID (keys %final_machine_group_map)
	{
		my $sql_str="select hostname from TLL_Nodelist where mgID=".$mgID;
		my $sth=$dbh->prepare($sql_str);
		if(defined($sth->execute()))
		{
			my($hostname);
			$sth->bind_columns(undef,\$hostname);
			while($sth->fetch())
			{
				push(@all_machines,$hostname);
			}
		}
		else
		{
			$json_str .= $dbh->errstr;
			print $json_str;
			exit 1;
		}	
		$sth->finish();		
	}	

	foreach my $machine (@all_machines)
	{
		# Get the admin name.
		my $admin_name = get_admin();
		if (length($admin_name) == 0)
		{
			$json_str .= "No LoadLeveler admin is available to run the reconfiguration on this node.\nRun the reconfiguration from the command line.";
			print $json_str;
			exit 1;
		}
		# Change user to the admin and run reconfig
		debug_log("command:".$cmd_name." ".$admin_name." -h ".$machine);
		my $out_str = `$cmd_name $admin_name -h $machine`;
		debug_log($out_str);
		
		if($? != 0)
		{
			debug_log($out_str);
			$json_str .= $out_str;
			print $json_str;
			exit 1;
		}
	}
	
}

$dbh->disconnect();
$json_str .= "OK";
print $json_str;
